#include "mainwindow.h"
#include "ui_mainwindow.h"

#include "aboutdialog.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    connect(ui->addBtn, SIGNAL(clicked()), SLOT(onAdd()));
    connect(ui->removeBtn, SIGNAL(clicked()), SLOT(onRemove()));

    connect(ui->actionQuit, SIGNAL(triggered()), qApp, SLOT(quit()));
    connect(ui->action_About, SIGNAL(triggered()), SLOT(onHelp()));
    connect(ui->newPathBtn, SIGNAL(clicked()), SLOT(onNewPath()));
    connect(ui->resizeBtn, SIGNAL(clicked()), SLOT(onResize()));
    connect(ui->filename, SIGNAL(textChanged(QString)), SLOT(onTextChanged(QString)));

    QDesktopWidget * desktop = QApplication::desktop();
    QSize windowSize = this->size();

    move((desktop->width() - windowSize.width()) / 2,(desktop->height() - windowSize.height()) / 2);

    mRegex = QRegExp(":|\\\\|/"); // Regex for directories separator (unix, mac or windows)
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::onAdd()
{
    QStringList files = QFileDialog::getOpenFileNames(this,
                                                      tr("Selectionnez des fichiers images"),
                                                      QDir::homePath(),
                                                      tr("Fichiers images (*.png *.jpg *.jpeg)"));
    if (files.isEmpty())
        return;

    foreach(QString file, files) {
        ui->pictureList->addItem(new QListWidgetItem(file));
    }
}

void MainWindow::onRemove()
{
    foreach (QListWidgetItem * item, ui->pictureList->selectedItems())
        delete item;
}

void MainWindow::onResize()
{
    int count = ui->pictureList->count();
    bool changeName = ui->changeFilename->isChecked();
    QString newName = ui->filename->text();

    if (count == 0)
    {
        QMessageBox::critical(this,
                              tr("Pas de fichier"),
                              tr("Vous n'avez pas ajout de fichier dans la liste."));
        return;
    }

    if (ui->newPath->text().isEmpty() && changeName == false)
    {
        if (QMessageBox::question(this,
                                  tr("Pas de rpertoire de destination"),
                                  tr("Attention ! \nVous n'avez pas indiqu de rpertoire de destination\n"
                                     "Vous allez dtruire les images d'origine.\n\n"
                                     "Voulez-vous continuer quand mme ?"),
                                  QMessageBox::Yes | QMessageBox::No, QMessageBox::Yes) == QMessageBox::No)
            return;
    }

    QProgressDialog progress(this);
    progress.setLabelText(tr("Miniaturisation de l'image"));
    progress.setRange(0, count);
    progress.show();

    int size = ui->horizontalSlider->value();
    QString newFilename;
    QString dirName = ui->newPath->text();
    QDir destDir(dirName);

    if (changeName == true && !newName.contains("%1")) {
        newName += "%1";
    }

    for(int i = 0; i < count; i++)
    {
        QString name;

        if (progress.wasCanceled())
            break;

        // Load image
        QString oldPath = ui->pictureList->item(i)->text();
        QImage image(oldPath);

        if (!image.isNull())
        {
            QImage newImage = image.scaled(size, size, Qt::KeepAspectRatio);
            QFileInfo info(oldPath);
            if (changeName == true)
            {
                name = newName.arg(QString().sprintf("%03d", i));
            }
            else
            {
                name = info.baseName();
            }
            if (dirName.isEmpty())
            {
                destDir = info.absoluteDir();
            }
            newFilename = destDir.filePath(QString("%1.%2").arg(name).arg(info.completeSuffix()));
            newImage.save(newFilename);
        }
        progress.setValue(i);
    }
    QMessageBox::information(this, tr("Termin"), tr("Le redimensionnement des images s'est effectu avec succs."));
}

void MainWindow::onHelp()
{
    AboutDialog(this).exec();
}

void MainWindow::onNewPath()
{
    QString path = QFileDialog::getExistingDirectory(this,
                                                     tr("Selectionnez un emplacement de destination"),
                                                     QDir::homePath());
    if (path.isEmpty())
        return;
    ui->newPath->setText(path);
}

void MainWindow::onTextChanged(QString s)
{
    if (s.contains(mRegex))
    {
        ui->filename->setText(s.remove(mRegex));
    }
}
